#include "internal/print_content.hpp"
#include "internal/storage_backend_serialized.hpp"
#include <iostream>
#include <getopt.h>
#include "libdbuspolicy1-private.h"

using namespace std;

static const struct option options[] = {
	{"system", no_argument, 0, 0 },
	{"session", no_argument, 0, 0},
	{0, 0, 0, 0}
};

static void print_help(const char *name) {
	cout << endl;
	cout << "usage: " << name << " [-i serialized filename] [-v]" << endl;
	cout << "       " << name << " {--system|--session} [-v]" << endl;
	cout << " -v - just verify, don't print anything" << endl;
	cout << " -x - print rule as xml format" << endl;
	cout << endl;
}

int main(int argc, char *argv[]) {
	std::string input_filename;
	int c;
	bool just_verify = false;
	bool xml_format = false;

	if (argc < 2) {
		print_help(argv[0]);
		return EXIT_FAILURE;
	}

	while (1) {
		int option_index;
		c = getopt_long(argc, argv, "i:vx", options, &option_index);
		if (c == -1)
			break;

		switch(c) {
		case 0:
			if (option_index == 0)
				input_filename = system_bus_conf_file_primary();
			else
				input_filename = session_bus_conf_file_primary();
			input_filename.append(".serialized");
			break;
		case 'i':
			input_filename = optarg;
			break;
		case 'v':
			just_verify = true;
			break;
		case 'x':
			xml_format = true;
			break;
		case '?':
			print_help(argv[0]);
			return EXIT_FAILURE;
		}
	}

	if (optind < argc)
		input_filename = argv[optind];

	if (input_filename.empty()) {
		cout << "No input filename" << endl;
		print_help(argv[0]);
		return EXIT_FAILURE;
	}

	tslog::init(tslog::ldp_log_level::DEFAULT);

	ldp_serialized::StorageBackendSerialized storage;

	if (!storage.init(input_filename.c_str(), true)) {
		if (just_verify)
			cout << input_filename << ": FAILED" << endl;
		return EXIT_FAILURE;
	}

	if (just_verify) {
		cout << input_filename << ": OK" << endl;
		return EXIT_SUCCESS;
	}

	storage.printContent(xml_format);

	return EXIT_SUCCESS;
}
