/*
 * Copyright (c) 2016-2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
#include "cynara.hpp"
#include <cstdlib>
#include <pthread.h>
#include <stdexcept>
#include <sys/types.h>
#include <unistd.h>

using namespace ldp_cynara;

bool Cynara::init() {
	if (!__inited) {
		int r = cynara_initialize(&__cynara, NULL);
		if (r != CYNARA_API_SUCCESS)
			return false;

		__session = cynara_session_from_pid(getpid());
		__inited = true;
	}
	return true;
}

static pthread_mutex_t __mutex = PTHREAD_MUTEX_INITIALIZER;
static Cynara c;

CynaraResult Cynara::check(const char* label, const std::string &privilege, uid_t uid) {
	const char* _label = "";
	CynaraResult ret = CynaraResult::DENY;
	if (label)
		_label = label;
	pthread_mutex_lock(&__mutex);
	if (!c.init()) {
		ret = CynaraResult::ERROR_INIT;
	} else {
		int r;
		try {
			r = cynara_check(c.__cynara, _label, c.__session, std::to_string(uid).c_str(), privilege.c_str());
			if (r == CYNARA_API_ACCESS_ALLOWED)
				ret = CynaraResult::ALLOW;
			else if (r == CYNARA_API_ACCESS_DENIED)
				ret = CynaraResult::DENY;
			else
				ret = CynaraResult::ERROR_CHECK;
		} catch (const std::bad_alloc &) {}
	}
	pthread_mutex_unlock(&__mutex);
	return ret;
}
