/*
 * Copyright (c) 2015-2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

/**
 * \file
 * \ingroup Implementation
 */


#ifndef GLOBAL_NODESTRUCT_HPP
#define GLOBAL_NODESTRUCT_HPP

/** Declare a global variable that is constructed (via gcc's __attribute__((constructor))) but never destroyed. */
#define DCL_NODESTRUCT_GLOBAL(TYPE,NAME)\
	namespace detail_NODESTRUCT_GLOBAL { alignas(TYPE) extern uint8_t NAME[sizeof(TYPE)]; }\
	/* awkwardly written to silence gcc's -Wstrict-aliasing because _Pragma("GCC diagnostic ignored \"-Wstrict-aliasing\"") is bugged */\
	static inline TYPE &NAME() { auto p = static_cast<void*>(&detail_NODESTRUCT_GLOBAL::NAME[0]); return *reinterpret_cast<TYPE*>(p); }\
	namespace detailInit_NODESTRUCT_GLOBAL { __attribute__((constructor(101))) static void NAME() { new(&::NAME()) TYPE(); } }

/** Define a global variable that is constructed (via gcc's __attribute__((constructor))) but never destroyed. */
#define DEF_NODESTRUCT_GLOBAL(TYPE,NAME)\
	namespace detail_NODESTRUCT_GLOBAL { alignas(TYPE) uint8_t NAME[sizeof(TYPE)]; }


#endif // GLOBAL_NODESTRUCT_HPP
