/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
#pragma once

#include "policy.hpp"
#include "fb_generated.h"

#include <memory>

namespace ldp_serialized {

class StorageBackendSerialized {
	class StorageBackendSerializedImpl;
	std::unique_ptr<StorageBackendSerializedImpl> pimpl;
public:
	StorageBackendSerialized();
	~StorageBackendSerialized();

	bool init(const char *filename, bool verify = false);
	bool init(const FB::File *file);
	bool initFromXML(const char *config_name);
	void release();

	void printContent(const bool xml_format = false) const;

	// Supported template parameters are:
	// MatchPolicyOwn, MatchPolicySend, MatchPolicyReceive
	// and - only for Contexts - MatchPolicyAccess
	template <typename T>
	ldp_xml_parser::DecisionItem getDecisionItemContextMandatory(const T &item) const;
	template <typename T>
	ldp_xml_parser::DecisionItem getDecisionItemContextDefault(const T &item) const;
	template <typename T>
	ldp_xml_parser::DecisionItem getDecisionItemUser(uid_t uid, const T &item) const;
	template <typename T>
	ldp_xml_parser::DecisionItem getDecisionItemGroup(gid_t gid, const T &item) const;

	// This works with T set to MatchItemOwn, MatchItemSend or MatchItemReceive
	// This is needed for filtering mapGroups. Check NaivePolicyChecker.
	template <typename T>
	bool existsPolicyForGroup(gid_t gid) const;

};

}
