#include "internal/include/fb_generated.h"
#include "internal/naive_policy_checker.hpp"
#include "internal/policy.hpp"
#include "internal/serializer.hpp"
#include "internal/storage_backend_serialized.hpp"
#include "internal/storage_backend_serialized.hpp"
#include "internal/tslog.hpp"
#include <map>

using namespace ldp_xml_parser;
using namespace ldp_serialized;

struct SignalTest {
	Decision expected_result;
	uid_t user;
	gid_t group;
	const char* label;
	const char* dest;
	const char* interface;
};

std::map<Decision, const char*> DECISIONS {
	{ Decision::ANY,   "ANY"   },
	{ Decision::ALLOW, "ALLOW" },
	{ Decision::DENY,  "DENY"  },
	{ Decision::CHECK, "CHECK" }
};

/**
 * This test set tests ability to parse xml db
 * and check signal call allowance in many use cases
 */
struct SignalTest signal_tests[]={
	(struct SignalTest){Decision::ALLOW,  0,    0, "test", "bli.bla.blubb test.test1 test.tes3", "/an/object/path"},
	(struct SignalTest){Decision::DENY,  5010, 0, "test", "bli.bla.blubb", "/an/object/path"},
};

void signalTest_print(const struct SignalTest* t, Decision result) {
	printf("uid = %lu, gid = %lu, label = %s, dest = %s, interface = %s, expected = %s, result = %s",
		   (unsigned long)t->user, (unsigned long)t->group, t->label, t->dest, t->interface, DECISIONS[t->expected_result], DECISIONS[result]);
}

template <typename DB>
bool signal_test(const DB &db) {
	unsigned  i = 0;
	bool flag = true;
	for (const auto &test : signal_tests) {
		KdbusBusNames names;
		MatchItemSend m_item(test.interface, NULL, NULL, ldp_xml_parser::MessageType::SIGNAL,
				names.addSpaceSeparatedNames(test.dest));

		auto ret = db.getDecisionItemContextMandatory(m_item);

		if (ret.getDecision() == Decision::ANY)
			ret = db.getDecisionItemUser(test.user, m_item);

		if (ret.getDecision() == Decision::ANY)
			ret = db.getDecisionItemGroup(test.group, m_item);

		if (ret.getDecision() == Decision::ANY)
			ret = db.getDecisionItemContextDefault(m_item);

		auto decision = ret.getDecision();

		if (test.expected_result != decision) {
			printf("[ERROR][%d] signal test failed: %s %s ", i, DECISIONS[test.expected_result], DECISIONS[decision]);
			signalTest_print(&test, decision);
			printf("\n");
			flag = false;
		}
	}
	return flag;
}

bool run_policy_db() {
	policy_checker_system().initDb("tests/default_allow/system.conf");
	auto &db = policy_checker_system().getPolicyDb();

	printf("POLICY_DB:\n");
	return signal_test(db);
}

bool run_fb() {
	Serializer serializer;
	size_t size;
	uint8_t *buff = serializer.serialize("tests/default_allow/system.conf", size);

	const FB::File *file = FB::GetFile(buff);

	StorageBackendSerialized storage;
	storage.init(file);

	printf("FLATBUFFERS:\n");
	return signal_test(storage);
}

bool run_xml() {
	return true;
}

bool run_tests() {
	return run_policy_db() && run_fb() && run_xml();
}

int main() {
	tslog::init();
	if (!run_tests())
		return -1;
return 0;
}
