#include "internal/naive_policy_checker.hpp"
#include "internal/tslog.hpp"

using ldp_xml_parser::MatchItemSend;
using ldp_xml_parser::DecisionResult;

struct SignalTest {
	DecisionResult expected_result;
	uid_t user;
	gid_t group;
	const char* label;
	const char* dest;
	const char* interface;
};
/**
 * This test set tests ability to parse xml db
 * and check signal call allowance in many use cases
 */
struct SignalTest signal_tests[]={
	{DecisionResult::ALLOW,    0, 0, "test", "bli.bla.blubb test.test1 test.tes3", "/an/object/path"},
	{DecisionResult::DENY,  5010, 0, "test", "bli.bla.blubb", "/an/object/path"},
};

std::map<DecisionResult, const char*> DECISIONS {
		{DecisionResult::CYNARA_ERROR, "CYNARA_ERROR"},
		{DecisionResult::DENY, "DENY"},
		{DecisionResult::ALLOW, "ALLOW"}};

void signalTest_print(struct SignalTest* t, DecisionResult result) {
	printf("uid = %lu, gid = %lu, label = %s, dest = %s, interface = %s, expected = %s, result = %s",
		   (unsigned long)t->user, (unsigned long)t->group, t->label, t->dest, t->interface,
		   DECISIONS[t->expected_result], DECISIONS[result]);
}

bool signal_test() {
	unsigned  i = 0;
	bool flag = true;
	policy_checker_system().initDb("tests/default_allow/system.conf");
	for (i = 0; i < sizeof(signal_tests)/sizeof(struct SignalTest); i++) {
		KdbusBusNames names;
		MatchItemSend itemSend(signal_tests[i].interface,
							   nullptr,
							   nullptr,
							   ldp_xml_parser::MessageType::SIGNAL,
							   names.addSpaceSeparatedNames(signal_tests[i].dest));

		auto decision = policy_checker_system().check(signal_tests[i].user,
													  signal_tests[i].group,
													  signal_tests[i].label,
													  itemSend);

		if (signal_tests[i].expected_result != decision) {
			printf("[ERROR][%d] signal test failed: %s %s ",
					i, DECISIONS[signal_tests[i].expected_result], DECISIONS[decision]);
			signalTest_print(&signal_tests[i], decision);
			printf("\n");
			flag = false;
		}
	}
	return flag;
}

int main() {
	tslog::init();
	if (!signal_test())
		return -1;
	return 0;
}
